@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 /**
  * SECTION:element-flacenc
@@ -22,13 +22,20 @@
  *
  * flacenc encodes FLAC streams.
  * <ulink url="http://flac.sourceforge.net/">FLAC</ulink>
- * is a Free Lossless Audio Codec.
+ * is a Free Lossless Audio Codec. FLAC audio can directly be written into
+ * a file, or embedded into containers such as oggmux or matroskamux.
  *
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- * gst-launch audiotestsrc num-buffers=100 ! flacenc ! filesink location=beep.flac
- * ]|
+ * gst-launch-1.0 audiotestsrc num-buffers=100 ! flacenc ! filesink location=beep.flac
+ * ]| Encode a short sine wave into FLAC
+ * |[
+ * gst-launch-1.0 cdparanoiasrc mode=continuous ! queue ! audioconvert ! flacenc ! filesink location=cd.flac
+ * ]| Rip a whole audio CD into a single FLAC file, with the track table saved as a CUE sheet inside the FLAC file
+ * |[
+ * gst-launch-1.0 cdparanoiasrc track=5 ! queue ! audioconvert ! flacenc ! filesink location=track5.flac
+ * ]| Rip track 5 of an audio CD and encode it losslessly to a FLAC file
  * </refsect2>
  */
 
@@ -91,9 +98,9 @@ static const GstAudioChannelPosition channel_positions[8][8] = {
 };
 
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
-#define FORMATS "{ S8LE, S16LE, S24LE, S24_32LE } "
+#define FORMATS "{ S24LE, S24_32LE, S16LE, S8 } "
 #else
-#define FORMATS "{ S8BE, S16BE, S24BE, S24_32BE } "
+#define FORMATS "{ S24BE, S24_32BE, S16BE, S8 } "
 #endif
 
 #define FLAC_SINK_CAPS                                    \
@@ -140,7 +147,9 @@ GST_DEBUG_CATEGORY_STATIC (flacenc_debug);
 
 #define gst_flac_enc_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstFlacEnc, gst_flac_enc, GST_TYPE_AUDIO_ENCODER,
-    G_IMPLEMENT_INTERFACE (GST_TYPE_TAG_SETTER, NULL));
+    G_IMPLEMENT_INTERFACE (GST_TYPE_TAG_SETTER, NULL)
+    G_IMPLEMENT_INTERFACE (GST_TYPE_TOC_SETTER, NULL)
+    );
 
 static gboolean gst_flac_enc_start (GstAudioEncoder * enc);
 static gboolean gst_flac_enc_stop (GstAudioEncoder * enc);
@@ -329,14 +338,6 @@ gst_flac_enc_class_init (GstFlacEncClass * klass)
           "parameters, use best", 0, FLAC__MAX_RICE_PARTITION_ORDER,
           flacenc_params[DEFAULT_QUALITY].rice_parameter_search_dist,
           G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
-
-  /**
-   * GstFlacEnc:padding
-   *
-   * Write a PADDING block with this length in bytes
-   *
-   * Since: 0.10.16
-   **/
   g_object_class_install_property (G_OBJECT_CLASS (klass),
       PROP_PADDING,
       g_param_spec_uint ("padding",
@@ -344,15 +345,6 @@ gst_flac_enc_class_init (GstFlacEncClass * klass)
           "Write a PADDING block with this length in bytes", 0, G_MAXUINT,
           DEFAULT_PADDING,
           G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
-
-  /**
-   * GstFlacEnc:seekpoints
-   *
-   * Write a SEEKTABLE block with a specific number of seekpoints
-   * or with a specific interval spacing.
-   *
-   * Since: 0.10.18
-   **/
   g_object_class_install_property (G_OBJECT_CLASS (klass),
       PROP_SEEKPOINTS,
       g_param_spec_int ("seekpoints",
@@ -415,6 +407,7 @@ gst_flac_enc_start (GstAudioEncoder * enc)
   flacenc->offset = 0;
   flacenc->eos = FALSE;
   flacenc->tags = gst_tag_list_new_empty ();
+  flacenc->toc = NULL;
 
   return TRUE;
 }
@@ -425,8 +418,11 @@ gst_flac_enc_stop (GstAudioEncoder * enc)
   GstFlacEnc *flacenc = GST_FLAC_ENC (enc);
 
   GST_DEBUG_OBJECT (enc, "stop");
-  gst_tag_list_free (flacenc->tags);
+  gst_tag_list_unref (flacenc->tags);
   flacenc->tags = NULL;
+  if (flacenc->toc)
+    gst_toc_unref (flacenc->toc);
+  flacenc->toc = NULL;
   if (FLAC__stream_encoder_get_state (flacenc->encoder) !=
       FLAC__STREAM_ENCODER_UNINITIALIZED) {
     flacenc->stopped = TRUE;
@@ -441,6 +437,9 @@ gst_flac_enc_stop (GstAudioEncoder * enc)
     if (flacenc->meta[2])
       FLAC__metadata_object_delete (flacenc->meta[2]);
 
+    if (flacenc->meta[3])
+      FLAC__metadata_object_delete (flacenc->meta[3]);
+
     g_free (flacenc->meta);
     flacenc->meta = NULL;
   }
@@ -449,6 +448,7 @@ gst_flac_enc_stop (GstAudioEncoder * enc)
   flacenc->headers = NULL;
 
   gst_tag_setter_reset_tags (GST_TAG_SETTER (enc));
+  gst_toc_setter_reset (GST_TOC_SETTER (enc));
 
   return TRUE;
 }
@@ -481,17 +481,107 @@ add_one_tag (const GstTagList * list, const gchar * tag, gpointer user_data)
   g_list_free (comments);
 }
 
+static gboolean
+add_cuesheet (const GstToc * toc, guint sample_rate,
+    FLAC__StreamMetadata * cuesheet)
+{
+  gint8 track_num = 0;
+  gint64 start, stop;
+  gchar *isrc = NULL;
+  const gchar *is_legal;
+  GList *list;
+  GstTagList *tags;
+  GstTocEntry *entry, *subentry = NULL;
+  FLAC__StreamMetadata_CueSheet *cs;
+  FLAC__StreamMetadata_CueSheet_Track *track;
+
+  cs = &cuesheet->data.cue_sheet;
+  if (!cs)
+    return FALSE;
+
+  /* check if the TOC entries is valid */
+  list = gst_toc_get_entries (toc);
+  entry = list->data;
+  if (gst_toc_entry_is_alternative (entry)) {
+    list = gst_toc_entry_get_sub_entries (entry);
+    while (list) {
+      subentry = list->data;
+      if (!gst_toc_entry_is_sequence (subentry))
+        return FALSE;
+      list = g_list_next (list);
+    }
+    list = gst_toc_entry_get_sub_entries (entry);
+  }
+  if (gst_toc_entry_is_sequence (entry)) {
+    while (list) {
+      entry = list->data;
+      if (!gst_toc_entry_is_sequence (entry))
+        return FALSE;
+      list = g_list_next (list);
+    }
+    list = gst_toc_get_entries (toc);
+  }
+
+  /* add tracks in cuesheet */
+  while (list) {
+    entry = list->data;
+    gst_toc_entry_get_start_stop_times (entry, &start, &stop);
+    tags = gst_toc_entry_get_tags (entry);
+    if (tags)
+      gst_tag_list_get_string (tags, GST_TAG_ISRC, &isrc);
+    track = FLAC__metadata_object_cuesheet_track_new ();
+    track->offset =
+        (FLAC__uint64) gst_util_uint64_scale_round (start, sample_rate,
+        GST_SECOND);
+    track->number = (FLAC__byte) track_num + 1;
+    if (isrc)
+      strcpy (track->isrc, isrc);
+    if (track->number <= 0)
+      return FALSE;
+    if (!FLAC__metadata_object_cuesheet_insert_track (cuesheet, track_num,
+            track, FALSE))
+      return FALSE;
+    if (!FLAC__metadata_object_cuesheet_track_insert_blank_index (cuesheet,
+            track_num, 0))
+      return FALSE;
+    track_num++;
+    list = g_list_next (list);
+  }
+
+  if (cs->num_tracks <= 0)
+    return FALSE;
+
+  /* add lead-out track in cuesheet */
+  track = FLAC__metadata_object_cuesheet_track_new ();
+  track->offset =
+      (FLAC__uint64) gst_util_uint64_scale_round (stop, sample_rate,
+      GST_SECOND);
+  track->number = 255;
+  if (!FLAC__metadata_object_cuesheet_insert_track (cuesheet, cs->num_tracks,
+          track, FALSE))
+    return FALSE;
+
+  /* check if the cuesheet is valid */
+  if (!FLAC__metadata_object_cuesheet_is_legal (cuesheet, FALSE, &is_legal)) {
+    g_warning ("%s\n", is_legal);
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
 static void
-gst_flac_enc_set_metadata (GstFlacEnc * flacenc, guint64 total_samples)
+gst_flac_enc_set_metadata (GstFlacEnc * flacenc, GstAudioInfo * info,
+    guint64 total_samples)
 {
   const GstTagList *user_tags;
   GstTagList *copy;
   gint entries = 1;
   gint n_images, n_preview_images;
-  GstAudioInfo *info =
-      gst_audio_encoder_get_audio_info (GST_AUDIO_ENCODER (flacenc));
+  FLAC__StreamMetadata *cuesheet;
 
   g_return_if_fail (flacenc != NULL);
+
   user_tags = gst_tag_setter_get_tag_list (GST_TAG_SETTER (flacenc));
   if ((flacenc->tags == NULL) && (user_tags == NULL)) {
     return;
@@ -502,62 +592,83 @@ gst_flac_enc_set_metadata (GstFlacEnc * flacenc, guint64 total_samples)
   n_preview_images = gst_tag_list_get_tag_size (copy, GST_TAG_PREVIEW_IMAGE);
 
   flacenc->meta =
-      g_new0 (FLAC__StreamMetadata *, 3 + n_images + n_preview_images);
+      g_new0 (FLAC__StreamMetadata *, 4 + n_images + n_preview_images);
 
   flacenc->meta[0] =
       FLAC__metadata_object_new (FLAC__METADATA_TYPE_VORBIS_COMMENT);
   gst_tag_list_foreach (copy, add_one_tag, flacenc);
 
+  if (!flacenc->toc)
+    flacenc->toc = gst_toc_setter_get_toc (GST_TOC_SETTER (flacenc));
+
+  if (flacenc->toc) {
+    cuesheet = FLAC__metadata_object_new (FLAC__METADATA_TYPE_CUESHEET);
+    if (add_cuesheet (flacenc->toc, GST_AUDIO_INFO_RATE (info), cuesheet)) {
+      flacenc->meta[entries] = cuesheet;
+      entries++;
+    } else {
+      FLAC__metadata_object_delete (cuesheet);
+      flacenc->meta[entries] = NULL;
+    }
+  }
+
   if (n_images + n_preview_images > 0) {
+    GstSample *sample;
     GstBuffer *buffer;
-#if 0
     GstCaps *caps;
-    GstStructure *structure;
+    const GstStructure *structure;
     GstTagImageType image_type = GST_TAG_IMAGE_TYPE_NONE;
-#endif
     gint i;
     GstMapInfo map;
 
     for (i = 0; i < n_images + n_preview_images; i++) {
       if (i < n_images) {
-        if (!gst_tag_list_get_buffer_index (copy, GST_TAG_IMAGE, i, &buffer))
+        if (!gst_tag_list_get_sample_index (copy, GST_TAG_IMAGE, i, &sample))
           continue;
       } else {
-        if (!gst_tag_list_get_buffer_index (copy, GST_TAG_PREVIEW_IMAGE,
-                i - n_images, &buffer))
+        if (!gst_tag_list_get_sample_index (copy, GST_TAG_PREVIEW_IMAGE,
+                i - n_images, &sample))
           continue;
       }
 
+      structure = gst_sample_get_info (sample);
+      caps = gst_sample_get_caps (sample);
+      if (!caps) {
+        GST_FIXME_OBJECT (flacenc, "Image tag without caps");
+        gst_sample_unref (sample);
+        continue;
+      }
+
       flacenc->meta[entries] =
           FLAC__metadata_object_new (FLAC__METADATA_TYPE_PICTURE);
 
-#if 0
-      caps = gst_buffer_get_caps (buffer);
-      structure = gst_caps_get_structure (caps, 0);
+      if (structure)
+        gst_structure_get (structure, "image-type", GST_TYPE_TAG_IMAGE_TYPE,
+            &image_type, NULL);
+      else
+        image_type = GST_TAG_IMAGE_TYPE_NONE;
 
-      gst_structure_get (structure, "image-type", GST_TYPE_TAG_IMAGE_TYPE,
-          &image_type, NULL);
       /* Convert to ID3v2 APIC image type */
       if (image_type == GST_TAG_IMAGE_TYPE_NONE)
         image_type = (i < n_images) ? 0x00 : 0x01;
       else
         image_type = image_type + 2;
-#endif
 
+      buffer = gst_sample_get_buffer (sample);
       gst_buffer_map (buffer, &map, GST_MAP_READ);
       FLAC__metadata_object_picture_set_data (flacenc->meta[entries],
           map.data, map.size, TRUE);
       gst_buffer_unmap (buffer, &map);
 
-#if 0
       /* FIXME: There's no way to set the picture type in libFLAC */
       flacenc->meta[entries]->data.picture.type = image_type;
+
+      structure = gst_caps_get_structure (caps, 0);
+
       FLAC__metadata_object_picture_set_mime_type (flacenc->meta[entries],
           (char *) gst_structure_get_name (structure), TRUE);
-      gst_caps_unref (caps);
-#endif
 
-      gst_buffer_unref (buffer);
+      gst_sample_unref (sample);
       entries++;
     }
   }
@@ -601,7 +712,7 @@ gst_flac_enc_set_metadata (GstFlacEnc * flacenc, guint64 total_samples)
           flacenc->meta, entries) != true)
     g_warning ("Dude, i'm already initialized!");
 
-  gst_tag_list_free (copy);
+  gst_tag_list_unref (copy);
 }
 
 static GstCaps *
@@ -623,7 +734,7 @@ gst_flac_enc_getcaps (GstAudioEncoder * enc, GstCaps * filter)
     g_value_init (&v_list, GST_TYPE_LIST);
     g_value_init (&v, G_TYPE_STRING);
 
-    g_value_set_static_string (&v, GST_AUDIO_NE (S8));
+    g_value_set_static_string (&v, "S8");
     gst_value_list_append_value (&v_list, &v);
     g_value_set_static_string (&v, GST_AUDIO_NE (S16));
     gst_value_list_append_value (&v_list, &v);
@@ -649,7 +760,7 @@ gst_flac_enc_getcaps (GstAudioEncoder * enc, GstCaps * filter)
         guint64 channel_mask;
 
         gst_audio_channel_positions_to_mask (channel_positions[i - 1], i,
-            &channel_mask);
+            FALSE, &channel_mask);
         gst_structure_set (s2, "channels", G_TYPE_INT, i, "channel-mask",
             GST_TYPE_BITMASK, channel_mask, NULL);
       }
@@ -661,7 +772,7 @@ gst_flac_enc_getcaps (GstAudioEncoder * enc, GstCaps * filter)
 
   GST_DEBUG_OBJECT (pad, "Return caps %" GST_PTR_FORMAT, ret);
 
-  caps = gst_audio_encoder_proxy_getcaps (enc, ret);
+  caps = gst_audio_encoder_proxy_getcaps (enc, ret, filter);
   gst_caps_unref (ret);
 
   return caps;
@@ -706,7 +817,6 @@ gst_flac_enc_set_format (GstAudioEncoder * enc, GstAudioInfo * info)
   GstFlacEnc *flacenc;
   guint64 total_samples = GST_CLOCK_TIME_NONE;
   FLAC__StreamEncoderInitStatus init_status;
-  GstCaps *caps;
 
   flacenc = GST_FLAC_ENC (enc);
 
@@ -715,14 +825,7 @@ gst_flac_enc_set_format (GstAudioEncoder * enc, GstAudioInfo * info)
       FLAC__STREAM_ENCODER_UNINITIALIZED)
     goto encoder_already_initialized;
 
-  caps = gst_caps_new_simple ("audio/x-flac",
-      "channels", G_TYPE_INT, GST_AUDIO_INFO_CHANNELS (info),
-      "rate", G_TYPE_INT, GST_AUDIO_INFO_RATE (info), NULL);
-
-  if (!gst_audio_encoder_set_output_format (enc, caps))
-    goto setting_src_caps_failed;
-
-  gst_caps_unref (caps);
+  /* delay setting output caps/format until we have all headers */
 
   gst_audio_get_channel_reorder_map (GST_AUDIO_INFO_CHANNELS (info),
       channel_positions[GST_AUDIO_INFO_CHANNELS (info) - 1], info->position,
@@ -732,7 +835,7 @@ gst_flac_enc_set_format (GstAudioEncoder * enc, GstAudioInfo * info)
       GST_AUDIO_ENCODER_SINK_PAD (enc));
 
   FLAC__stream_encoder_set_bits_per_sample (flacenc->encoder,
-      GST_AUDIO_INFO_WIDTH (info));
+      GST_AUDIO_INFO_DEPTH (info));
   FLAC__stream_encoder_set_sample_rate (flacenc->encoder,
       GST_AUDIO_INFO_RATE (info));
   FLAC__stream_encoder_set_channels (flacenc->encoder,
@@ -742,7 +845,7 @@ gst_flac_enc_set_format (GstAudioEncoder * enc, GstAudioInfo * info)
     FLAC__stream_encoder_set_total_samples_estimate (flacenc->encoder,
         MIN (total_samples, G_GUINT64_CONSTANT (0x0FFFFFFFFF)));
 
-  gst_flac_enc_set_metadata (flacenc, total_samples);
+  gst_flac_enc_set_metadata (flacenc, info, total_samples);
 
   /* callbacks clear to go now;
    * write callbacks receives headers during init */
@@ -763,13 +866,6 @@ encoder_already_initialized:
     g_warning ("flac already initialized -- fixme allow this");
     return FALSE;
   }
-setting_src_caps_failed:
-  {
-    GST_DEBUG_OBJECT (flacenc,
-        "Couldn't set caps on source pad: %" GST_PTR_FORMAT, caps);
-    gst_caps_unref (caps);
-    return FALSE;
-  }
 failed_to_initialize:
   {
     GST_ELEMENT_ERROR (flacenc, LIBRARY, INIT, (NULL),
@@ -1012,23 +1108,7 @@ gst_flac_enc_process_stream_headers (GstFlacEnc * enc)
 push_headers:
   gst_audio_encoder_set_output_format (GST_AUDIO_ENCODER (enc), caps);
 
-  /* push header buffers; update caps, so when we push the first buffer the
-   * negotiated caps will change to caps that include the streamheader field */
-  for (l = enc->headers; l != NULL; l = l->next) {
-    GstBuffer *buf;
-
-    buf = GST_BUFFER (l->data);
-    GST_LOG_OBJECT (enc,
-        "Pushing header buffer, size %" G_GSIZE_FORMAT " bytes",
-        gst_buffer_get_size (buf));
-#if 0
-    GST_MEMDUMP_OBJECT (enc, "header buffer", GST_BUFFER_DATA (buf),
-        GST_BUFFER_SIZE (buf));
-#endif
-    ret = gst_pad_push (GST_AUDIO_ENCODER_SRC_PAD (enc), buf);
-    l->data = NULL;
-  }
-  g_list_free (enc->headers);
+  gst_audio_encoder_set_headers (GST_AUDIO_ENCODER (enc), enc->headers);
   enc->headers = NULL;
 
   gst_caps_unref (caps);
@@ -1079,8 +1159,7 @@ gst_flac_enc_write_callback (const FLAC__StreamEncoder * encoder,
     ret = gst_pad_push (GST_AUDIO_ENCODER_SRC_PAD (flacenc), outbuf);
   } else {
     /* regular frame data, pass to base class */
-    GST_LOG ("Pushing buffer: ts=%" GST_TIME_FORMAT ", samples=%u, size=%u, "
-        "pos=%" G_GUINT64_FORMAT, GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)),
+    GST_LOG ("Pushing buffer: samples=%u, size=%u, pos=%" G_GUINT64_FORMAT,
         samples, (guint) bytes, flacenc->offset);
     ret = gst_audio_encoder_finish_frame (GST_AUDIO_ENCODER (flacenc),
         outbuf, samples);
@@ -1116,11 +1195,13 @@ gst_flac_enc_sink_event (GstAudioEncoder * enc, GstEvent * event)
 {
   GstFlacEnc *flacenc;
   GstTagList *taglist;
+  GstToc *toc;
   gboolean ret = FALSE;
 
   flacenc = GST_FLAC_ENC (enc);
 
-  GST_DEBUG ("Received %s event on sinkpad", GST_EVENT_TYPE_NAME (event));
+  GST_DEBUG ("Received %s event on sinkpad, %" GST_PTR_FORMAT,
+      GST_EVENT_TYPE_NAME (event), event);
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_EOS:
@@ -1137,6 +1218,17 @@ gst_flac_enc_sink_event (GstAudioEncoder * enc, GstEvent * event)
       }
       ret = GST_AUDIO_ENCODER_CLASS (parent_class)->sink_event (enc, event);
       break;
+    case GST_EVENT_TOC:
+      gst_event_parse_toc (event, &toc, NULL);
+      if (toc) {
+        if (flacenc->toc != toc) {
+          if (flacenc->toc)
+            gst_toc_unref (flacenc->toc);
+          flacenc->toc = toc;
+        }
+      }
+      ret = GST_AUDIO_ENCODER_CLASS (parent_class)->sink_event (enc, event);
+      break;
     default:
       ret = GST_AUDIO_ENCODER_CLASS (parent_class)->sink_event (enc, event);
       break;
@@ -1177,6 +1269,7 @@ gst_flac_enc_handle_frame (GstAudioEncoder * enc, GstBuffer * buffer)
 
   if (G_UNLIKELY (!buffer)) {
     if (flacenc->eos) {
+      GST_DEBUG_OBJECT (flacenc, "finish encoding");
       FLAC__stream_encoder_finish (flacenc->encoder);
     } else {
       /* can't handle intermittent draining/resyncing */
@@ -1194,6 +1287,8 @@ gst_flac_enc_handle_frame (GstAudioEncoder * enc, GstBuffer * buffer)
   data = g_malloc (samples * sizeof (FLAC__int32));
 
   samples /= channels;
+  GST_LOG_OBJECT (flacenc, "processing %d samples, %d channels", samples,
+      channels);
   if (width == 8) {
     gint8 *indata = (gint8 *) map.data;
 
@@ -1232,7 +1327,7 @@ gst_flac_enc_handle_frame (GstAudioEncoder * enc, GstBuffer * buffer)
   gst_buffer_unmap (buffer, &map);
 
   res = FLAC__stream_encoder_process_interleaved (flacenc->encoder,
-      (const FLAC__int32 *) data, samples / channels);
+      (const FLAC__int32 *) data, samples);
 
   g_free (data);
 
