@@ -41,6 +41,7 @@
 #include <gst/gst.h>
 #include <gst/base/gstbasesink.h>
 #include "gstsouphttpclientsink.h"
+#include "gstsouputils.h"
 
 #include <gst/glib-compat-private.h>
 
@@ -93,10 +94,12 @@ enum
   PROP_PROXY_ID,
   PROP_PROXY_PW,
   PROP_COOKIES,
-  PROP_SESSION
+  PROP_SESSION,
+  PROP_SOUP_LOG_LEVEL
 };
 
 #define DEFAULT_USER_AGENT           "GStreamer souphttpclientsink "
+#define DEFAULT_SOUP_LOG_LEVEL       SOUP_LOGGER_LOG_NONE
 
 /* pad templates */
 
@@ -168,6 +171,19 @@ gst_soup_http_client_sink_class_init (GstSoupHttpClientSinkClass * klass)
   g_object_class_install_property (gobject_class, PROP_COOKIES,
       g_param_spec_boxed ("cookies", "Cookies", "HTTP request cookies",
           G_TYPE_STRV, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+ /**
+   * GstSoupHttpClientSink::http-log-level:
+   *
+   * If set and > 0, captures and dumps HTTP session data as
+   * log messages if log level >= GST_LEVEL_TRACE
+   *
+   * Since: 1.4
+   */
+  g_object_class_install_property (gobject_class, PROP_SOUP_LOG_LEVEL,
+      g_param_spec_enum ("http-log-level", "HTTP log level",
+          "Set log level for soup's HTTP session log",
+          SOUP_TYPE_LOGGER_LOG_LEVEL, DEFAULT_SOUP_LOG_LEVEL,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&gst_soup_http_client_sink_sink_template));
@@ -214,6 +230,7 @@ gst_soup_http_client_sink_init (GstSoupHttpClientSink * souphttpsink)
   souphttpsink->proxy_pw = NULL;
   souphttpsink->prop_session = NULL;
   souphttpsink->timeout = 1;
+  souphttpsink->log_level = DEFAULT_SOUP_LOG_LEVEL;
   proxy = g_getenv ("http_proxy");
   if (proxy && !gst_soup_http_client_sink_set_proxy (souphttpsink, proxy)) {
     GST_WARNING_OBJECT (souphttpsink,
@@ -227,6 +244,9 @@ gst_soup_http_client_sink_init (GstSoupHttpClientSink * souphttpsink)
 static void
 gst_soup_http_client_sink_reset (GstSoupHttpClientSink * souphttpsink)
 {
+  g_list_free_full (souphttpsink->queued_buffers,
+      (GDestroyNotify) gst_buffer_unref);
+  souphttpsink->queued_buffers = NULL;
   g_free (souphttpsink->reason_phrase);
   souphttpsink->reason_phrase = NULL;
   souphttpsink->status_code = 0;
@@ -316,6 +336,9 @@ gst_soup_http_client_sink_set_property (GObject * object, guint property_id,
       g_strfreev (souphttpsink->cookies);
       souphttpsink->cookies = g_strdupv (g_value_get_boxed (value));
       break;
+    case PROP_SOUP_LOG_LEVEL:
+      souphttpsink->log_level = g_value_get_enum (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -368,6 +391,9 @@ gst_soup_http_client_sink_get_property (GObject * object, guint property_id,
     case PROP_COOKIES:
       g_value_set_boxed (value, g_strdupv (souphttpsink->cookies));
       break;
+    case PROP_SOUP_LOG_LEVEL:
+      g_value_set_enum (value, souphttpsink->log_level);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -419,6 +445,7 @@ gst_soup_http_client_sink_set_caps (GstBaseSink * sink, GstCaps * caps)
   const GValue *value_array;
   int i, n;
 
+  GST_DEBUG_OBJECT (souphttpsink, "new stream headers set");
   structure = gst_caps_get_structure (caps, 0);
   value_array = gst_structure_get_value (structure, "streamheader");
   if (value_array) {
@@ -514,13 +541,14 @@ gst_soup_http_client_sink_start (GstBaseSink * sink)
         souphttpsink->user_agent, SOUP_SESSION_TIMEOUT, souphttpsink->timeout,
         NULL);
 
-    //soup_session_add_feature (souphttpsink->session,
-    //    SOUP_SESSION_FEATURE (soup_logger_new (SOUP_LOGGER_LOG_BODY, 100)));
-
     g_signal_connect (souphttpsink->session, "authenticate",
         G_CALLBACK (authenticate), souphttpsink);
   }
 
+  /* Set up logging */
+  gst_soup_util_log_setup (souphttpsink->session, souphttpsink->log_level,
+      GST_ELEMENT (souphttpsink));
+
   return TRUE;
 }
 
@@ -612,6 +640,7 @@ send_message_locked (GstSoupHttpClientSink * souphttpsink)
 
   /* If the URI went away, drop all these buffers */
   if (souphttpsink->location == NULL) {
+    GST_DEBUG_OBJECT (souphttpsink, "URI went away, dropping queued buffers");
     free_buffer_list (souphttpsink->queued_buffers);
     souphttpsink->queued_buffers = NULL;
     return;
@@ -625,10 +654,13 @@ send_message_locked (GstSoupHttpClientSink * souphttpsink)
       GstBuffer *buffer = g->data;
       GstMapInfo map;
 
-      /* FIXME, lifetime of the buffer? */
+      GST_DEBUG_OBJECT (souphttpsink, "queueing stream headers");
       gst_buffer_map (buffer, &map, GST_MAP_READ);
+      /* Stream headers are updated whenever ::set_caps is called, so there's
+       * no guarantees about their lifetime and we ask libsoup to copy them 
+       * into the message body with SOUP_MEMORY_COPY. */
       soup_message_body_append (souphttpsink->message->request_body,
-          SOUP_MEMORY_STATIC, map.data, map.size);
+          SOUP_MEMORY_COPY, map.data, map.size);
       n += map.size;
       gst_buffer_unmap (buffer, &map);
     }
@@ -639,10 +671,13 @@ send_message_locked (GstSoupHttpClientSink * souphttpsink)
     if (!GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_HEADER)) {
       GstMapInfo map;
 
-      /* FIXME, lifetime of the buffer? */
       gst_buffer_map (buffer, &map, GST_MAP_READ);
+      /* Queued buffers are only freed in the next iteration of the mainloop
+       * after the message body has been written out, so we don't need libsoup
+       * to copy those while appending to the body. However, if the buffer is
+       * used elsewhere, it should be copied. Hence, SOUP_MEMORY_TEMPORARY. */
       soup_message_body_append (souphttpsink->message->request_body,
-          SOUP_MEMORY_STATIC, map.data, map.size);
+          SOUP_MEMORY_TEMPORARY, map.data, map.size);
       n += map.size;
       gst_buffer_unmap (buffer, &map);
     }
@@ -658,6 +693,8 @@ send_message_locked (GstSoupHttpClientSink * souphttpsink)
   }
 
   if (n == 0) {
+    GST_DEBUG_OBJECT (souphttpsink,
+        "total size of buffers queued is 0, freeing everything");
     free_buffer_list (souphttpsink->queued_buffers);
     souphttpsink->queued_buffers = NULL;
     g_object_unref (souphttpsink->message);
@@ -736,6 +773,7 @@ gst_soup_http_client_sink_render (GstBaseSink * sink, GstBuffer * buffer)
         g_list_append (souphttpsink->queued_buffers, gst_buffer_ref (buffer));
 
     if (wake) {
+      GST_DEBUG_OBJECT (souphttpsink, "setting callback for new buffers");
       source = g_idle_source_new ();
       g_source_set_callback (source, (GSourceFunc) (send_message),
           souphttpsink, NULL);
