@@ -269,6 +269,7 @@ gst_speex_enc_stop (GstAudioEncoder * benc)
     enc->state = NULL;
   }
   speex_bits_destroy (&enc->bits);
+  speex_bits_set_bit_buffer (&enc->bits, NULL, 0);
   gst_tag_list_unref (enc->tags);
   enc->tags = NULL;
 
@@ -658,6 +659,7 @@ _gst_caps_set_buffer_array (GstCaps * caps, const gchar * field,
 
     buf = va_arg (va, GstBuffer *);
   }
+  va_end (va);
 
   gst_structure_set_value (structure, field, &array);
   g_value_unset (&array);
