@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 /**
@@ -106,7 +106,7 @@ static gboolean gst_vp8_dec_start (GstVideoDecoder * decoder);
 static gboolean gst_vp8_dec_stop (GstVideoDecoder * decoder);
 static gboolean gst_vp8_dec_set_format (GstVideoDecoder * decoder,
     GstVideoCodecState * state);
-static gboolean gst_vp8_dec_reset (GstVideoDecoder * decoder, gboolean hard);
+static gboolean gst_vp8_dec_flush (GstVideoDecoder * decoder);
 static GstFlowReturn gst_vp8_dec_handle_frame (GstVideoDecoder * decoder,
     GstVideoCodecFrame * frame);
 static gboolean gst_vp8_dec_decide_allocation (GstVideoDecoder * decoder,
@@ -176,7 +176,7 @@ gst_vp8_dec_class_init (GstVP8DecClass * klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_vp8_dec_sink_template));
 
-  gst_element_class_set_metadata (element_class,
+  gst_element_class_set_static_metadata (element_class,
       "On2 VP8 Decoder",
       "Codec/Decoder/Video",
       "Decode VP8 video streams", "David Schleef <ds@entropywave.com>, "
@@ -184,7 +184,7 @@ gst_vp8_dec_class_init (GstVP8DecClass * klass)
 
   base_video_decoder_class->start = GST_DEBUG_FUNCPTR (gst_vp8_dec_start);
   base_video_decoder_class->stop = GST_DEBUG_FUNCPTR (gst_vp8_dec_stop);
-  base_video_decoder_class->reset = GST_DEBUG_FUNCPTR (gst_vp8_dec_reset);
+  base_video_decoder_class->flush = GST_DEBUG_FUNCPTR (gst_vp8_dec_flush);
   base_video_decoder_class->set_format =
       GST_DEBUG_FUNCPTR (gst_vp8_dec_set_format);
   base_video_decoder_class->handle_frame =
@@ -205,6 +205,8 @@ gst_vp8_dec_init (GstVP8Dec * gst_vp8_dec)
   gst_vp8_dec->post_processing_flags = DEFAULT_POST_PROCESSING_FLAGS;
   gst_vp8_dec->deblocking_level = DEFAULT_DEBLOCKING_LEVEL;
   gst_vp8_dec->noise_level = DEFAULT_NOISE_LEVEL;
+
+  gst_video_decoder_set_needs_format (decoder, TRUE);
 }
 
 static void
@@ -292,13 +294,16 @@ gst_vp8_dec_stop (GstVideoDecoder * base_video_decoder)
     gst_video_codec_state_unref (gst_vp8_dec->output_state);
     gst_vp8_dec->output_state = NULL;
   }
+
   if (gst_vp8_dec->input_state) {
     gst_video_codec_state_unref (gst_vp8_dec->input_state);
     gst_vp8_dec->input_state = NULL;
   }
+
   if (gst_vp8_dec->decoder_inited)
     vpx_codec_destroy (&gst_vp8_dec->decoder);
   gst_vp8_dec->decoder_inited = FALSE;
+
   return TRUE;
 }
 
@@ -308,21 +313,32 @@ gst_vp8_dec_set_format (GstVideoDecoder * decoder, GstVideoCodecState * state)
   GstVP8Dec *gst_vp8_dec = GST_VP8_DEC (decoder);
 
   GST_DEBUG_OBJECT (gst_vp8_dec, "set_format");
+
+  if (gst_vp8_dec->decoder_inited)
+    vpx_codec_destroy (&gst_vp8_dec->decoder);
   gst_vp8_dec->decoder_inited = FALSE;
 
-  if (gst_vp8_dec->input_state)
+  if (gst_vp8_dec->output_state) {
+    gst_video_codec_state_unref (gst_vp8_dec->output_state);
+    gst_vp8_dec->output_state = NULL;
+  }
+
+  if (gst_vp8_dec->input_state) {
     gst_video_codec_state_unref (gst_vp8_dec->input_state);
+    gst_vp8_dec->input_state = NULL;
+  }
+
   gst_vp8_dec->input_state = gst_video_codec_state_ref (state);
 
   return TRUE;
 }
 
 static gboolean
-gst_vp8_dec_reset (GstVideoDecoder * base_video_decoder, gboolean hard)
+gst_vp8_dec_flush (GstVideoDecoder * base_video_decoder)
 {
   GstVP8Dec *decoder;
 
-  GST_DEBUG_OBJECT (base_video_decoder, "reset");
+  GST_DEBUG_OBJECT (base_video_decoder, "flush");
 
   decoder = GST_VP8_DEC (base_video_decoder);
 
@@ -330,10 +346,7 @@ gst_vp8_dec_reset (GstVideoDecoder * base_video_decoder, gboolean hard)
     gst_video_codec_state_unref (decoder->output_state);
     decoder->output_state = NULL;
   }
-  if (hard && decoder->input_state) {
-    gst_video_codec_state_unref (decoder->input_state);
-    decoder->input_state = NULL;
-  }
+
   if (decoder->decoder_inited)
     vpx_codec_destroy (&decoder->decoder);
   decoder->decoder_inited = FALSE;
@@ -412,10 +425,16 @@ open_codec (GstVP8Dec * dec, GstVideoCodecFrame * frame)
 
   gst_buffer_unmap (frame->input_buffer, &minfo);
 
-  if (status != VPX_CODEC_OK || !stream_info.is_kf) {
+  if (status != VPX_CODEC_OK) {
+    GST_WARNING_OBJECT (dec, "VPX preprocessing error: %s",
+        gst_vpx_error_name (status));
+    gst_video_decoder_finish_frame (GST_VIDEO_DECODER (dec), frame);
+    return GST_FLOW_CUSTOM_SUCCESS_1;
+  }
+  if (!stream_info.is_kf) {
     GST_WARNING_OBJECT (dec, "No keyframe, skipping");
     gst_video_decoder_finish_frame (GST_VIDEO_DECODER (dec), frame);
-    return GST_FLOW_OK;
+    return GST_FLOW_CUSTOM_SUCCESS_1;
   }
 
   g_assert (dec->output_state == NULL);
@@ -478,13 +497,20 @@ gst_vp8_dec_handle_frame (GstVideoDecoder * decoder, GstVideoCodecFrame * frame)
   long decoder_deadline = 0;
   GstClockTimeDiff deadline;
   GstMapInfo minfo;
+  GstVideoInfo *info;
+  GstVideoCodecState *new_output_state;
 
-  GST_DEBUG_OBJECT (decoder, "handle_frame");
+  GST_LOG_OBJECT (decoder, "handle_frame");
 
   dec = GST_VP8_DEC (decoder);
 
-  if (!dec->decoder_inited)
+  if (!dec->decoder_inited) {
     ret = open_codec (dec, frame);
+    if (ret == GST_FLOW_CUSTOM_SUCCESS_1)
+      return GST_FLOW_OK;
+    else if (ret != GST_FLOW_OK)
+      return ret;
+  }
 
   deadline = gst_video_decoder_get_max_decode_time (decoder, frame);
   if (deadline < 0) {
@@ -506,18 +532,45 @@ gst_vp8_dec_handle_frame (GstVideoDecoder * decoder, GstVideoCodecFrame * frame)
   gst_buffer_unmap (frame->input_buffer, &minfo);
 
   if (status) {
-    GST_ELEMENT_ERROR (decoder, LIBRARY, ENCODE,
-        ("Failed to decode frame"), ("%s", gst_vpx_error_name (status)));
-    return GST_FLOW_ERROR;
+    GST_VIDEO_DECODER_ERROR (decoder, 1, LIBRARY, ENCODE,
+        ("Failed to decode frame"), ("%s", gst_vpx_error_name (status)), ret);
+    return ret;
   }
 
   img = vpx_codec_get_frame (&dec->decoder, &iter);
   if (img) {
+    if (img->fmt != VPX_IMG_FMT_I420) {
+      vpx_img_free (img);
+      GST_ELEMENT_ERROR (decoder, LIBRARY, ENCODE,
+          ("Failed to decode frame"), ("Unsupported color format %d",
+              img->fmt));
+      return GST_FLOW_ERROR;
+    }
+
     if (deadline < 0) {
       GST_LOG_OBJECT (dec, "Skipping late frame (%f s past deadline)",
           (double) -deadline / GST_SECOND);
       gst_video_decoder_drop_frame (decoder, frame);
     } else {
+      info = &dec->output_state->info;
+      if (GST_VIDEO_INFO_WIDTH (info) != img->d_w
+          || GST_VIDEO_INFO_HEIGHT (info) != img->d_h) {
+        GST_DEBUG_OBJECT (dec,
+            "Changed output resolution was %d x %d now is got %u x %u (display %u x %u)",
+            GST_VIDEO_INFO_WIDTH (info), GST_VIDEO_INFO_HEIGHT (info), img->w,
+            img->h, img->d_w, img->d_h);
+
+        new_output_state =
+            gst_video_decoder_set_output_state (GST_VIDEO_DECODER (dec),
+            GST_VIDEO_FORMAT_I420, img->d_w, img->d_h, dec->output_state);
+        if (dec->output_state) {
+          gst_video_codec_state_unref (dec->output_state);
+        }
+        dec->output_state = new_output_state;
+        /* No need to call negotiate() here, it will be automatically called
+         * by allocate_output_frame() below */
+      }
+
       ret = gst_video_decoder_allocate_output_frame (decoder, frame);
 
       if (ret == GST_FLOW_OK) {
