@@ -19,7 +19,7 @@
 
 #include "qtdemux_types.h"
 #include "qtdemux_dump.h"
-#include "qtdemux_fourcc.h"
+#include "fourcc.h"
 
 static const QtNodeType qt_node_types[] = {
   {FOURCC_moov, "movie", QT_FLAG_CONTAINER,},
@@ -174,8 +174,14 @@ static const QtNodeType qt_node_types[] = {
   {FOURCC_avcC, "AV codec configuration container", 0},
   {FOURCC_avc1, "AV codec configuration v1", 0},
   {FOURCC_avc3, "AV codec configuration v3", 0},
+  {FOURCC_mp4s, "VOBSUB codec configuration", 0},
+  {FOURCC_hvc1, "HEVC codec configuration", 0},
+  {FOURCC_hev1, "HEVC codec configuration", 0},
+  {FOURCC_hvcC, "HEVC codec configuration container", 0},
   {FOURCC_tfdt, "Track fragment decode time", 0, qtdemux_dump_tfdt},
   {FOURCC_chap, "Chapter Reference"},
+  {FOURCC_btrt, "Bitrate information", 0},
+  {FOURCC_frma, "Audio codec format", 0},
   {0, "unknown", 0,},
 };
 
