@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifndef __GST_RTP_H264_PAY_H__
@@ -59,7 +59,7 @@ struct _GstRtpH264Pay
   GstRTPBasePayload payload;
 
   guint profile;
-  GList *sps, *pps;
+  GPtrArray *sps, *pps;
 
   GstH264StreamFormat stream_format;
   GstH264Alignment alignment;
@@ -74,6 +74,11 @@ struct _GstRtpH264Pay
   guint spspps_interval;
   gboolean send_spspps;
   GstClockTime last_spspps;
+
+  /* TRUE if the next NALU processed should have the DELTA_UNIT flag */
+  gboolean delta_unit;
+  /* TRUE if the next NALU processed should have the DISCONT flag */
+  gboolean discont;
 };
 
 struct _GstRtpH264PayClass
