@@ -570,19 +570,16 @@ gst_rtp_theora_depay_process (GstRTPBaseDepayload * depayload, GstBuffer * buf)
       break;
   }
 
-  g_free (to_free);
-
   if (rtptheoradepay->needs_keyframe)
     goto request_keyframe;
 
+out:
+no_output:
+
   gst_rtp_buffer_unmap (&rtp);
+  g_free (to_free);
   return NULL;
 
-no_output:
-  {
-    gst_rtp_buffer_unmap (&rtp);
-    return NULL;
-  }
   /* ERORRS */
 switch_failed:
   {
@@ -599,8 +596,7 @@ packet_short:
 ignore_reserved:
   {
     GST_WARNING_OBJECT (rtptheoradepay, "reserved TDT ignored");
-    gst_rtp_buffer_unmap (&rtp);
-    return NULL;
+    goto out;
   }
 length_short:
   {
@@ -621,8 +617,7 @@ request_config:
         gst_event_new_custom (GST_EVENT_CUSTOM_UPSTREAM,
             gst_structure_new ("GstForceKeyUnit",
                 "all-headers", G_TYPE_BOOLEAN, TRUE, NULL)));
-    gst_rtp_buffer_unmap (&rtp);
-    return NULL;
+    goto out;
   }
 request_keyframe:
   {
@@ -630,8 +625,7 @@ request_keyframe:
     gst_pad_push_event (GST_RTP_BASE_DEPAYLOAD_SINKPAD (depayload),
         gst_event_new_custom (GST_EVENT_CUSTOM_UPSTREAM,
             gst_structure_new_empty ("GstForceKeyUnit")));
-    gst_rtp_buffer_unmap (&rtp);
-    return NULL;
+    goto out;
   }
 }
 
