@@ -29,10 +29,6 @@
  * It can be combined with rtp payload encoders to implement RTP streaming.
  */
 
-/* FIXME 0.11: suppress warnings for deprecated API such as GValueArray
- * with newer GLib versions (>= 2.31.0) */
-#define GLIB_DISABLE_DEPRECATION_WARNINGS
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -309,7 +305,7 @@ gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass)
           " FALSE = disable", DEFAULT_LOOP,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
-   * GstMultiUDPSink::force-ipv4
+   * GstMultiUDPSink::force-ipv4:
    *
    * Force the use of an IPv4 socket.
    *
@@ -326,12 +322,10 @@ gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass)
           -1, 63, DEFAULT_QOS_DSCP,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
-   * GstMultiUDPSink::send-duplicates
+   * GstMultiUDPSink::send-duplicates:
    *
    * When a host/port pair is added mutliple times, send the packet to the host
    * multiple times as well.
-   *
-   * Since: 0.10.26
    */
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SEND_DUPLICATES,
       g_param_spec_boolean ("send-duplicates", "Send Duplicates",
@@ -358,7 +352,8 @@ gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass)
 
   gst_element_class_set_static_metadata (gstelement_class, "UDP packet sender",
       "Sink/Network",
-      "Send data over the network via UDP",
+      "Send data over the network via UDP to one or multiple recipients "
+      "which can be added or removed at runtime using action signals",
       "Wim Taymans <wim.taymans@gmail.com>");
 
   gstbasesink_class->render = gst_multiudpsink_render;
@@ -531,20 +526,20 @@ gst_multiudpsink_render (GstBaseSink * bsink, GstBuffer * buffer)
   gint num, no_clients;
   GError *err = NULL;
 
-  sink = GST_MULTIUDPSINK (bsink);
+  sink = GST_MULTIUDPSINK_CAST (bsink);
 
   n_mem = gst_buffer_n_memory (buffer);
   if (n_mem == 0)
     goto no_data;
 
-  /* allocated on the stack, the max number of memory blocks is limited so this
-   * should not cause stack overflows */
+  /* pre-allocated, the max number of memory blocks is limited so this
+   * should not cause overflows */
   vec = sink->vec;
   map = sink->map;
 
   size = 0;
   for (i = 0; i < n_mem; i++) {
-    mem = gst_buffer_get_memory (buffer, i);
+    mem = gst_buffer_peek_memory (buffer, i);
     gst_memory_map (mem, &map[i], GST_MAP_READ);
 
     vec[i].buffer = map[i].data;
@@ -618,10 +613,8 @@ gst_multiudpsink_render (GstBaseSink * bsink, GstBuffer * buffer)
   g_mutex_unlock (&sink->client_lock);
 
   /* unmap all memory again */
-  for (i = 0; i < n_mem; i++) {
+  for (i = 0; i < n_mem; i++)
     gst_memory_unmap (map[i].memory, &map[i]);
-    gst_memory_unref (map[i].memory);
-  }
 
   GST_LOG_OBJECT (sink, "sent %" G_GSIZE_FORMAT " bytes to %d (of %d) clients",
       size, num, no_clients);
@@ -639,10 +632,8 @@ flushing:
     g_clear_error (&err);
 
     /* unmap all memory */
-    for (i = 0; i < n_mem; i++) {
+    for (i = 0; i < n_mem; i++)
       gst_memory_unmap (map[i].memory, &map[i]);
-      gst_memory_unref (map[i].memory);
-    }
 
     return GST_FLOW_FLUSHING;
   }
@@ -1141,12 +1132,18 @@ gst_multiudpsink_start (GstBaseSink * bsink)
 #ifdef SO_BINDTODEVICE
   if (sink->multi_iface) {
     if (sink->used_socket) {
-      setsockopt (g_socket_get_fd (sink->used_socket), SOL_SOCKET,
-          SO_BINDTODEVICE, sink->multi_iface, strlen (sink->multi_iface));
+      if (setsockopt (g_socket_get_fd (sink->used_socket), SOL_SOCKET,
+              SO_BINDTODEVICE, sink->multi_iface,
+              strlen (sink->multi_iface)) < 0)
+        GST_WARNING_OBJECT (sink, "setsockopt SO_BINDTODEVICE failed: %s",
+            strerror (errno));
     }
     if (sink->used_socket_v6) {
-      setsockopt (g_socket_get_fd (sink->used_socket_v6), SOL_SOCKET,
-          SO_BINDTODEVICE, sink->multi_iface, strlen (sink->multi_iface));
+      if (setsockopt (g_socket_get_fd (sink->used_socket_v6), SOL_SOCKET,
+              SO_BINDTODEVICE, sink->multi_iface,
+              strlen (sink->multi_iface)) < 0)
+        GST_WARNING_OBJECT (sink, "setsockopt SO_BINDTODEVICE failed (v6): %s",
+            strerror (errno));
     }
   }
 #endif
@@ -1326,9 +1323,8 @@ gst_multiudpsink_remove (GstMultiUDPSink * sink, const gchar * host, gint port)
   client->refcount--;
   if (client->refcount == 0) {
     GInetSocketAddress *saddr = G_INET_SOCKET_ADDRESS (client->addr);
+    GSocketFamily family = g_socket_address_get_family (client->addr);
     GInetAddress *addr = g_inet_socket_address_get_address (saddr);
-    GSocketFamily family =
-        g_socket_address_get_family (G_SOCKET_ADDRESS (saddr));
     GSocket *socket;
 
     /* Select socket to send from for this address */
