@@ -2,7 +2,7 @@
  * OSX video sink
  * Copyright (C) 2004-6 Zaheer Abbas Merali <zaheerabbas at merali dot org>
  * Copyright (C) 2007,2008,2009 Pioneers of the Inevitable <songbird@songbirdnest.com>
- * 
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
  * License as published by the Free Software Foundation; either
@@ -20,7 +20,7 @@
  *
  * The development of this code was made possible due to the involvement of
  * Pioneers of the Inevitable, the creators of the Songbird Music player.
- * 
+ *
  */
 
 /**
@@ -29,16 +29,12 @@
  * The OSXVideoSink renders video frames to a MacOSX window. The video output
  * must be directed to a window embedded in an existing NSApp.
  *
- * When the NSView to be embedded is created an element #GstMessage with a 
- * name of 'have-ns-view' will be created and posted on the bus. 
- * The pointer to the NSView to embed will be in the 'nsview' field of that 
- * message. The application MUST handle this message and embed the view
- * appropriately.
  */
 
 #include "config.h"
 #include <gst/video/videooverlay.h>
 #include <gst/video/navigation.h>
+#include <gst/video/video.h>
 
 #include "osxvideosink.h"
 #include <unistd.h>
@@ -47,11 +43,9 @@
 GST_DEBUG_CATEGORY (gst_debug_osx_video_sink);
 #define GST_CAT_DEFAULT gst_debug_osx_video_sink
 
-#ifdef RUN_NS_APP_THREAD
 #include <pthread.h>
-extern  void _CFRunLoopSetCurrent(CFRunLoopRef rl);
+extern void _CFRunLoopSetCurrent (CFRunLoopRef rl);
 extern pthread_t _CFMainPThread;
-#endif
 
 
 
@@ -111,12 +105,8 @@ run_ns_app_loop (void) {
   NSAutoreleasePool *pool =[[NSAutoreleasePool alloc] init];
   NSDate *pollTime = nil;
 
-#ifdef RUN_NS_APP_THREAD
   /* when running the loop in a thread we want to sleep as long as possible */
   pollTime = [NSDate distantFuture];
-#else
-  pollTime = [NSDate distantPast];
-#endif
 
   do {
       event = [NSApp nextEventMatchingMask:NSAnyEventMask untilDate:pollTime
@@ -196,7 +186,6 @@ gst_osx_video_sink_run_cocoa_loop (GstOSXVideoSink * sink )
   }
 
   if (sink_class->ns_app_thread == NULL) {
-#ifdef RUN_NS_APP_THREAD
     /* run the main runloop in a separate thread */
 
     /* override [NSThread isMainThread] with our own implementation so that we can
@@ -217,12 +206,6 @@ gst_osx_video_sink_run_cocoa_loop (GstOSXVideoSink * sink )
     g_mutex_lock (&_run_loop_mutex);
     g_cond_wait (&_run_loop_cond, &_run_loop_mutex);
     g_mutex_unlock (&_run_loop_mutex);
-#else
-  /* assume that there is a GMainLoop and iterate the main runloop from there
-   */
-    sink_class->cocoa_timeout = g_timeout_add (10,
-        (GSourceFunc) run_ns_app_loop, NULL);
-#endif
   }
 
   g_mutex_unlock (&_run_loop_check_mutex);
@@ -231,10 +214,6 @@ gst_osx_video_sink_run_cocoa_loop (GstOSXVideoSink * sink )
 static void
 gst_osx_video_sink_stop_cocoa_loop (GstOSXVideoSink * osxvideosink)
 {
-#ifndef RUN_NS_APP_THREAD
-  if (sink_class->cocoa_timeout)
-    g_source_remove(sink_klass->cocoa_timeout);
-#endif
 }
 
 /* This function handles osx window creation */
@@ -244,8 +223,6 @@ gst_osx_video_sink_osxwindow_create (GstOSXVideoSink * osxvideosink, gint width,
 {
   NSRect rect;
   GstOSXWindow *osxwindow = NULL;
-  GstStructure *s;
-  GstMessage *msg;
   gboolean res = TRUE;
   NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
 
@@ -268,51 +245,29 @@ gst_osx_video_sink_osxwindow_create (GstOSXVideoSink * osxvideosink, gint width,
   rect.size.height = (float) osxwindow->height;
   osxwindow->gstview =[[GstGLView alloc] initWithFrame:rect];
 
-  s = gst_structure_new ("have-ns-view",
-     "nsview", G_TYPE_POINTER, osxwindow->gstview,
-     nil);
-
-  msg = gst_message_new_element (GST_OBJECT (osxvideosink), s);
-  gst_element_post_message (GST_ELEMENT (osxvideosink), msg);
-
-  GST_INFO_OBJECT (osxvideosink, "'have-ns-view' message sent");
 
   gst_osx_video_sink_run_cocoa_loop (osxvideosink);
   [osxwindow->gstview setMainThread:sink_class->ns_app_thread];
 
-  /* check if have-ns-view was handled and osxwindow->gstview was added to a
-   * superview
-   */
-  if ([osxwindow->gstview haveSuperview] == NO) {
-    /* have-ns-view wasn't handled, post prepare-xwindow-id */
-    if (osxvideosink->superview == NULL) {
-      GST_INFO_OBJECT (osxvideosink, "emitting prepare-xwindow-id");
-      gst_video_overlay_prepare_window_handle (GST_VIDEO_OVERLAY (osxvideosink));
-    }
+  if (osxvideosink->superview == NULL) {
+    GST_INFO_OBJECT (osxvideosink, "emitting prepare-xwindow-id");
+    gst_video_overlay_prepare_window_handle (GST_VIDEO_OVERLAY (osxvideosink));
+  }
 
-    if (osxvideosink->superview != NULL) {
-      /* prepare-xwindow-id was handled, we have the superview in
-       * osxvideosink->superview. We now add osxwindow->gstview to the superview
-       * from the main thread
-       */
-      GST_INFO_OBJECT (osxvideosink, "we have a superview, adding our view to it");
-      gst_osx_video_sink_call_from_main_thread(osxvideosink, osxwindow->gstview,
-          @selector(addToSuperview:), osxvideosink->superview, NO);
+  if (osxvideosink->superview != NULL) {
+    /* prepare-xwindow-id was handled, we have the superview in
+     * osxvideosink->superview. We now add osxwindow->gstview to the superview
+     * from the main thread
+     */
+    GST_INFO_OBJECT (osxvideosink, "we have a superview, adding our view to it");
+    gst_osx_video_sink_call_from_main_thread(osxvideosink, osxwindow->gstview,
+        @selector(addToSuperview:), osxvideosink->superview, NO);
 
-    } else {
-      if (osxvideosink->embed) {
-        /* the view wasn't added to a superview. It's possible that the
-         * application handled have-ns-view, stored our view internally and is
-         * going to add it to a superview later (webkit does that now).
-         */
-        GST_INFO_OBJECT (osxvideosink, "no superview");
-      } else {
-        gst_osx_video_sink_call_from_main_thread(osxvideosink,
-          osxvideosink->osxvideosinkobject,
-          @selector(createInternalWindow), nil, YES);
-        GST_INFO_OBJECT (osxvideosink, "No superview, creating an internal window.");
-      }
-    }
+  } else {
+    gst_osx_video_sink_call_from_main_thread(osxvideosink,
+      osxvideosink->osxvideosinkobject,
+      @selector(createInternalWindow), nil, YES);
+    GST_INFO_OBJECT (osxvideosink, "No superview, creating an internal window.");
   }
   [osxwindow->gstview setNavigation: GST_NAVIGATION(osxvideosink)];
   [osxvideosink->osxwindow->gstview setKeepAspectRatio: osxvideosink->keep_par];
@@ -357,7 +312,7 @@ gst_osx_video_sink_osxwindow_resize (GstOSXVideoSink * osxvideosink,
 
   /* Directly resize the underlying view */
   GST_DEBUG_OBJECT (osxvideosink, "Calling setVideoSize on %p", osxwindow->gstview);
-  gst_osx_video_sink_call_from_main_thread(osxvideosink, object,
+  gst_osx_video_sink_call_from_main_thread (osxvideosink, object,
       @selector(resize), (id)nil, YES);
 
   [pool release];
@@ -391,6 +346,9 @@ gst_osx_video_sink_setcaps (GstBaseSink * bsink, GstCaps * caps)
 
   gst_osx_video_sink_osxwindow_resize (osxvideosink, osxvideosink->osxwindow,
       video_width, video_height);
+
+  gst_video_info_from_caps (&osxvideosink->info, caps);
+
   result = TRUE;
 
 beach:
@@ -407,7 +365,7 @@ gst_osx_video_sink_change_state (GstElement * element,
 
   osxvideosink = GST_OSX_VIDEO_SINK (element);
 
-  GST_DEBUG_OBJECT (osxvideosink, "%s => %s", 
+  GST_DEBUG_OBJECT (osxvideosink, "%s => %s",
         gst_element_state_get_name(GST_STATE_TRANSITION_CURRENT (transition)),
         gst_element_state_get_name(GST_STATE_TRANSITION_NEXT (transition)));
 
@@ -487,9 +445,8 @@ gst_osx_video_sink_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case ARG_EMBED:
-      osxvideosink->embed = g_value_get_boolean(value);
       g_warning ("The \"embed\" property of osxvideosink is deprecated and "
-          "will be removed in the near future. Use the GstVideoOverlay "
+          "has no effect anymore. Use the GstVideoOverlay "
           "instead.");
       break;
     case ARG_FORCE_PAR:
@@ -516,7 +473,7 @@ gst_osx_video_sink_get_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case ARG_EMBED:
-      g_value_set_boolean (value, osxvideosink->embed);
+      g_value_set_boolean (value, FALSE);
       break;
     case ARG_FORCE_PAR:
       g_value_set_boolean (value, osxvideosink->keep_par);
@@ -527,6 +484,14 @@ gst_osx_video_sink_get_property (GObject * object, guint prop_id,
   }
 }
 
+static gboolean
+gst_osx_video_sink_propose_allocation (GstBaseSink * base_sink, GstQuery * query)
+{
+    gst_query_add_allocation_meta (query,
+        GST_VIDEO_META_API_TYPE, NULL);
+
+    return TRUE;
+}
 
 static void
 gst_osx_video_sink_init (GstOSXVideoSink * sink)
@@ -588,12 +553,13 @@ gst_osx_video_sink_class_init (GstOSXVideoSinkClass * klass)
   gstbasesink_class->set_caps = gst_osx_video_sink_setcaps;
   gstbasesink_class->preroll = gst_osx_video_sink_show_frame;
   gstbasesink_class->render = gst_osx_video_sink_show_frame;
+  gstbasesink_class->propose_allocation = gst_osx_video_sink_propose_allocation;
   gstelement_class->change_state = gst_osx_video_sink_change_state;
 
   /**
    * GstOSXVideoSink:embed
    *
-   * Set to #TRUE if you are embedding the video window in an application.
+   * For ABI comatibility onyl, do not use
    *
    **/
 
@@ -793,7 +759,6 @@ gst_osx_video_sink_get_type (void)
 -(void) createInternalWindow
 {
   GstOSXWindow *osxwindow = osxvideosink->osxwindow;
-  ProcessSerialNumber psn;
   NSRect rect;
   unsigned int mask;
 
@@ -810,19 +775,6 @@ gst_osx_video_sink_get_type (void)
   rect.size.width = (float) osxwindow->width;
   rect.size.height = (float) osxwindow->height;
 
-#ifndef RUN_NS_APP_THREAD
-  if (!osxvideosink->app_started) {
-    [NSApplication sharedApplication];
-    [NSApp finishLaunching];
-    osxvideosink->app_started = TRUE;
-  }
-#endif
-
-  if (!GetCurrentProcess(&psn)) {
-      TransformProcessType(&psn, kProcessTransformToForegroundApplication);
-      SetFrontProcess(&psn);
-  }
-
   osxwindow->win =[[[GstOSXVideoSinkWindow alloc]
                        initWithContentNSRect: rect
                        styleMask: mask
@@ -830,20 +782,18 @@ gst_osx_video_sink_get_type (void)
                        defer: NO
                        screen: nil] retain];
   GST_DEBUG("VideoSinkWindow created, %p", osxwindow->win);
-  [osxwindow->win makeKeyAndOrderFront:NSApp];
+  [osxwindow->win orderFrontRegardless];
   osxwindow->gstview =[osxwindow->win gstView];
   [osxwindow->win setDelegate:[[GstWindowDelegate alloc]
       initWithSink:osxvideosink]];
 
 }
 
-#ifdef RUN_NS_APP_THREAD
 + (BOOL) isMainThread
 {
   /* FIXME: ideally we should return YES only for ->ns_app_thread here */
   return YES;
 }
-#endif
 
 - (void) setView: (NSView*)view
 {
@@ -898,30 +848,56 @@ gst_osx_video_sink_get_type (void)
 
 - (void) showFrame: (GstBufferObject *) object
 {
-  GstMapInfo info;
-  guint8 *viewdata;
+  GstVideoFrame frame;
+  guint8 *data, *readp, *writep;
+  gint i, active_width, stride;
+  guint8 *texture_buffer;
   NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
   GstBuffer *buf = object->buf;
 
   GST_OBJECT_LOCK (osxvideosink);
-  if (osxvideosink->osxwindow != NULL)
-  {
-    gst_buffer_map (buf, &info, GST_MAP_READ);
-    viewdata = (guint8 *) [osxvideosink->osxwindow->gstview getTextureBuffer];
+  if (osxvideosink->osxwindow == NULL)
+      goto no_window;
 
-    if (G_UNLIKELY (viewdata == NULL)) {
-      GST_ELEMENT_ERROR (osxvideosink, RESOURCE, WRITE,
-        ("Could not get a texture buffer"), (NULL));
-    } else {
-      memcpy (viewdata, info.data, info.size);
-      [osxvideosink->osxwindow->gstview displayTexture];
-      gst_buffer_unmap (buf, &info);
-    }
+  texture_buffer = (guint8 *) [osxvideosink->osxwindow->gstview getTextureBuffer];
+  if (G_UNLIKELY (texture_buffer == NULL))
+      goto no_texture_buffer;
+
+  if (!gst_video_frame_map (&frame, &osxvideosink->info, buf, GST_MAP_READ))
+      goto no_map;
+
+  data = readp = GST_VIDEO_FRAME_PLANE_DATA (&frame, 0);
+  stride = GST_VIDEO_FRAME_PLANE_STRIDE (&frame, 0);
+  writep = texture_buffer;
+  active_width = GST_VIDEO_SINK_WIDTH (osxvideosink) * sizeof (short);
+  for (i = 0; i < GST_VIDEO_SINK_HEIGHT (osxvideosink); i++) {
+      memcpy (writep, readp, active_width);
+      writep += active_width;
+      readp += stride;
   }
+  [osxvideosink->osxwindow->gstview displayTexture];
+
+  gst_video_frame_unmap (&frame);
+
+out:
   GST_OBJECT_UNLOCK (osxvideosink);
   [object release];
 
   [pool release];
+  return;
+
+no_map:
+  GST_WARNING_OBJECT (osxvideosink, "couldn't map frame");
+  goto out;
+
+no_window:
+  GST_WARNING_OBJECT (osxvideosink, "not showing frame since we have no window (!?)");
+  goto out;
+
+no_texture_buffer:
+  GST_ELEMENT_ERROR (osxvideosink, RESOURCE, WRITE, (NULL),
+          ("the texture buffer is NULL"));
+  goto out;
 }
 
 -(void) destroy
@@ -951,7 +927,6 @@ gst_osx_video_sink_get_type (void)
   [pool release];
 }
 
-#ifdef RUN_NS_APP_THREAD
 -(void) nsAppThread
 {
   NSAutoreleasePool *pool;
@@ -981,7 +956,6 @@ gst_osx_video_sink_get_type (void)
 
   [pool release];
 }
-#endif
 
 -(void) checkMainRunLoop
 {
@@ -1012,7 +986,7 @@ plugin_init (GstPlugin * plugin)
 {
 
   if (!gst_element_register (plugin, "osxvideosink",
-          GST_RANK_PRIMARY, GST_TYPE_OSX_VIDEO_SINK))
+          GST_RANK_MARGINAL, GST_TYPE_OSX_VIDEO_SINK))
     return FALSE;
 
   GST_DEBUG_CATEGORY_INIT (gst_debug_osx_video_sink, "osxvideosink", 0,
