@@ -154,13 +154,17 @@ test_basic (const gchar * elem_name, const gchar * sink2, int count,
           gst_event_new_stream_start ("stream2")));
 
   gst_caps_set_simple (sinkcaps,
-      "payload", G_TYPE_INT, 98, "seqnum-base", G_TYPE_UINT, 100,
-      "clock-base", G_TYPE_UINT, 1000, "ssrc", G_TYPE_UINT, 66, NULL);
+      "payload", G_TYPE_INT, 98, "seqnum-offset", G_TYPE_UINT, 100,
+      "timestamp-offset", G_TYPE_UINT, 1000, "ssrc", G_TYPE_UINT, 66, NULL);
   caps = gst_caps_new_simple ("application/x-rtp",
       "payload", G_TYPE_INT, 98, "clock-rate", G_TYPE_INT, 3,
-      "seqnum-base", G_TYPE_UINT, 56, "clock-base", G_TYPE_UINT, 57,
+      "seqnum-offset", G_TYPE_UINT, 56, "timestamp-offset", G_TYPE_UINT, 57,
       "ssrc", G_TYPE_UINT, 66, NULL);
   fail_unless (gst_pad_set_caps (src1, caps));
+  gst_caps_unref (caps);
+
+  caps = gst_pad_peer_query_caps (sink, NULL);
+  fail_if (gst_caps_is_empty (caps));
 
   gst_segment_init (&segment, GST_FORMAT_TIME);
   segment.start = 100000;
@@ -226,7 +230,7 @@ basic_check_cb (GstPad * pad, int i)
   fail_unless (buffers && g_list_length (buffers) == 1);
 
   gst_rtp_buffer_map (buffers->data, GST_MAP_READ, &rtpbuffer);
-  fail_unless (gst_rtp_buffer_get_ssrc (&rtpbuffer) == 55);
+  fail_unless (gst_rtp_buffer_get_ssrc (&rtpbuffer) == 66);
   fail_unless (gst_rtp_buffer_get_timestamp (&rtpbuffer) ==
       200 - 57 + 1000 + i);
   fail_unless (gst_rtp_buffer_get_seq (&rtpbuffer) == 100 + 1 + i);
@@ -260,7 +264,7 @@ lock_check_cb (GstPad * pad, int i)
 
     fail_unless (buffers && g_list_length (buffers) == 1);
     gst_rtp_buffer_map (buffers->data, GST_MAP_READ, &rtpbuffer);
-    fail_unless (gst_rtp_buffer_get_ssrc (&rtpbuffer) == 55);
+    fail_unless (gst_rtp_buffer_get_ssrc (&rtpbuffer) == 66);
     fail_unless (gst_rtp_buffer_get_timestamp (&rtpbuffer) ==
         200 - 57 + 1000 + i);
     fail_unless (gst_rtp_buffer_get_seq (&rtpbuffer) == 100 + 1 + i);
